from http import HTTPStatus

from django.urls import reverse
from django.utils.http import urlencode

import pytest

from allauth.idp.oidc.models import Token


@pytest.mark.parametrize("method", ["GET", "POST"])
@pytest.mark.parametrize(
    "post_logout_redirect_uri,state,expected_location",
    [
        (None, None, "/"),
        ("https://rp.client/logged-out", None, "https://rp.client/logged-out"),
        (
            "https://rp.client/logged-out",
            "mystate",
            "https://rp.client/logged-out?state=mystate",
        ),
        ("http://no-http.org/please", None, "/"),
    ],
)
def test_logout_while_anonymous(
    method, client, post_logout_redirect_uri, state, expected_location
):
    params = {}
    if post_logout_redirect_uri:
        params["post_logout_redirect_uri"] = post_logout_redirect_uri
    if state:
        params["state"] = state
    if method == "GET":
        query = None
        if params:
            query = urlencode(params)
        resp = client.get(reverse("idp:oidc:logout") + (f"?{query}" if query else ""))
    else:
        resp = client.post(reverse("idp:oidc:logout"), data=params)
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"] == expected_location


@pytest.mark.parametrize("method", ["GET", "POST"])
def test_logout_must_ask(auth_client, settings, method):
    settings.IDP_OIDC_RP_INITIATED_LOGOUT_ASKS_FOR_OP_LOGOUT = False
    params = {}
    if method == "GET":
        query = None
        if params:
            query = urlencode(params)
        resp = auth_client.get(
            reverse("idp:oidc:logout") + (f"?{query}" if query else "")
        )
    else:
        resp = auth_client.post(reverse("idp:oidc:logout"), data=params)
    assert resp.status_code == HTTPStatus.OK


@pytest.mark.parametrize(
    "csrfmiddlewaretoken, status_code",
    [(None, HTTPStatus.OK), ("", HTTPStatus.FORBIDDEN), ("hack", HTTPStatus.FORBIDDEN)],
)
def test_rp_cannot_bypass(auth_client, csrfmiddlewaretoken, status_code):
    auth_client.enforce_csrf_checks = True
    auth_client.handler.enforce_csrf_checks = True
    params = {
        # Try a POST that answers the question right away...
        "action": "logout",
    }
    if csrfmiddlewaretoken is not None:
        params["csrfmiddlewaretoken"] = "hack"
    resp = auth_client.post(reverse("idp:oidc:logout"), data=params)
    assert resp.status_code == status_code


@pytest.mark.parametrize("method", ["GET", "POST"])
def test_logout_without_asking(
    auth_client,
    user,
    settings,
    method,
    oidc_client,
    id_token_generator,
    access_token_generator,
    refresh_token_factory,
):
    id_token_hint = id_token_generator(oidc_client, user)
    access_token, access_token_instance = access_token_generator(oidc_client, user)
    refresh_token, refresh_token_instance = refresh_token_factory(
        user=user, client=oidc_client
    )
    settings.IDP_OIDC_RP_INITIATED_LOGOUT_ASKS_FOR_OP_LOGOUT = False
    params = {
        "id_token_hint": id_token_hint,
        "post_logout_redirect_uri": "https://rp.client/logged-out",
    }
    if method == "GET":
        query = None
        if params:
            query = urlencode(params)
        resp = auth_client.get(
            reverse("idp:oidc:logout") + (f"?{query}" if query else "")
        )
    else:
        resp = auth_client.post(reverse("idp:oidc:logout"), data=params)
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"] == "https://rp.client/logged-out"

    assert not Token.objects.filter(pk=access_token_instance.pk).exists()
    assert not Token.objects.filter(pk=refresh_token_instance.pk).exists()
    resp = auth_client.get(reverse("account_email"))
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"].startswith(reverse("account_login"))



from http import HTTPStatus

from django.test import TestCase

from allauth.socialaccount.providers.trello.provider import TrelloProvider
from tests.apps.socialaccount.base import OAuthTestsMixin
from tests.mocking import MockedResponse


class TrelloTests(OAuthTestsMixin, TestCase):
    provider_id = TrelloProvider.id

    def get_mocked_response(self):
        return [
            MockedResponse(
                HTTPStatus.OK,
                r"""
{"id": "123", "email": "raymond.penners@example.com", "username": "pennersr", "name": "Raymond"}
        """,
            ),
        ]  # noqa

    def get_expected_to_str(self):
        return "pennersr"



from http import HTTPStatus
from unittest.mock import ANY

from django.contrib.auth import get_user_model

import pytest

from allauth.account import app_settings
from allauth.account.authentication import AUTHENTICATION_METHODS_SESSION_KEY
from allauth.headless.constants import Flow
from allauth.mfa.models import Authenticator


def test_passkey_login(
    client, passkey, webauthn_authentication_bypass, headless_reverse
):
    with webauthn_authentication_bypass(passkey) as credential:
        resp = client.get(headless_reverse("headless:mfa:login_webauthn"))
        assert "request_options" in resp.json()["data"]
        resp = client.post(
            headless_reverse("headless:mfa:login_webauthn"),
            data={"credential": credential},
            content_type="application/json",
        )
    data = resp.json()
    assert data["data"]["user"]["id"] == passkey.user_id


def test_passkey_login_get_options(client, headless_client, headless_reverse, db):
    resp = client.get(headless_reverse("headless:mfa:login_webauthn"))
    data = resp.json()
    meta = {}
    if headless_client == "app":
        meta = {
            "meta": {"session_token": ANY},
        }
    assert data == {
        "status": HTTPStatus.OK,
        "data": {"request_options": {"publicKey": ANY}},
        **meta,
    }


def test_reauthenticate(
    auth_client,
    passkey,
    user_with_recovery_codes,
    webauthn_authentication_bypass,
    headless_reverse,
):
    # View recovery codes, confirm webauthn reauthentication is an option
    resp = auth_client.get(headless_reverse("headless:mfa:manage_recovery_codes"))
    assert resp.status_code == HTTPStatus.UNAUTHORIZED
    assert Flow.MFA_REAUTHENTICATE in [
        flow["id"] for flow in resp.json()["data"]["flows"]
    ]

    # Get request options
    with webauthn_authentication_bypass(passkey):
        resp = auth_client.get(headless_reverse("headless:mfa:reauthenticate_webauthn"))
        data = resp.json()
        assert data["status"] == HTTPStatus.OK
        assert data["data"]["request_options"] == ANY

    # Reauthenticate
    with webauthn_authentication_bypass(passkey) as credential:
        resp = auth_client.post(
            headless_reverse("headless:mfa:reauthenticate_webauthn"),
            data={"credential": credential},
            content_type="application/json",
        )
        assert resp.status_code == HTTPStatus.OK
    resp = auth_client.get(headless_reverse("headless:mfa:manage_recovery_codes"))
    assert resp.status_code == HTTPStatus.OK


def test_update_authenticator(
    auth_client, headless_reverse, passkey, reauthentication_bypass
):
    data = {"id": passkey.pk, "name": "Renamed!"}
    resp = auth_client.put(
        headless_reverse("headless:mfa:manage_webauthn"),
        data=data,
        content_type="application/json",
    )
    # Reauthentication required
    assert resp.status_code == HTTPStatus.UNAUTHORIZED
    with reauthentication_bypass():
        resp = auth_client.put(
            headless_reverse("headless:mfa:manage_webauthn"),
            data=data,
            content_type="application/json",
        )
    assert resp.status_code == HTTPStatus.OK
    passkey.refresh_from_db()
    assert passkey.wrap().name == "Renamed!"


def test_delete_authenticator(
    auth_client, headless_reverse, passkey, reauthentication_bypass
):
    data = {"authenticators": [passkey.pk]}
    resp = auth_client.delete(
        headless_reverse("headless:mfa:manage_webauthn"),
        data=data,
        content_type="application/json",
    )
    # Reauthentication required
    assert resp.status_code == HTTPStatus.UNAUTHORIZED
    with reauthentication_bypass():
        resp = auth_client.delete(
            headless_reverse("headless:mfa:manage_webauthn"),
            data=data,
            content_type="application/json",
        )
    assert resp.status_code == HTTPStatus.OK
    assert not Authenticator.objects.filter(pk=passkey.pk).exists()


@pytest.mark.parametrize("email_verified", [False, True])
def test_add_authenticator(
    user,
    auth_client,
    headless_reverse,
    webauthn_registration_bypass,
    reauthentication_bypass,
    email_verified,
):
    resp = auth_client.get(headless_reverse("headless:mfa:manage_webauthn"))
    # Reauthentication required
    assert (
        resp.status_code == HTTPStatus.UNAUTHORIZED
        if email_verified
        else HTTPStatus.CONFLICT
    )

    with reauthentication_bypass():
        resp = auth_client.get(headless_reverse("headless:mfa:manage_webauthn"))
        if email_verified:
            assert resp.status_code == HTTPStatus.OK
            data = resp.json()
            assert data["data"]["creation_options"] == ANY
        else:
            assert resp.status_code == HTTPStatus.CONFLICT

        with webauthn_registration_bypass(user, False) as credential:
            resp = auth_client.post(
                headless_reverse("headless:mfa:manage_webauthn"),
                data={"credential": credential},
                content_type="application/json",
            )
            webauthn_count = Authenticator.objects.filter(
                type=Authenticator.Type.WEBAUTHN, user=user
            ).count()
            if email_verified:
                assert resp.status_code == HTTPStatus.OK
                assert webauthn_count == 1
            else:
                assert resp.status_code == HTTPStatus.CONFLICT
                assert webauthn_count == 0


def test_2fa_login(
    client,
    user,
    user_password,
    passkey,
    webauthn_authentication_bypass,
    headless_reverse,
):
    resp = client.post(
        headless_reverse("headless:account:login"),
        data={
            "username": user.username,
            "password": user_password,
        },
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.UNAUTHORIZED
    data = resp.json()
    pending_flows = [f for f in data["data"]["flows"] if f.get("is_pending")]
    assert len(pending_flows) == 1
    pending_flow = pending_flows[0]
    assert pending_flow == {
        "id": "mfa_authenticate",
        "is_pending": True,
        "types": ["webauthn"],
    }
    with webauthn_authentication_bypass(passkey) as credential:
        resp = client.get(headless_reverse("headless:mfa:authenticate_webauthn"))
        assert "request_options" in resp.json()["data"]
        resp = client.post(
            headless_reverse("headless:mfa:authenticate_webauthn"),
            data={"credential": credential},
            content_type="application/json",
        )
    data = resp.json()
    assert resp.status_code == HTTPStatus.OK
    assert data["data"]["user"]["id"] == passkey.user_id
    assert client.headless_session()[AUTHENTICATION_METHODS_SESSION_KEY] == [
        {"method": "password", "at": ANY, "username": passkey.user.username},
        {"method": "mfa", "at": ANY, "id": ANY, "type": Authenticator.Type.WEBAUTHN},
    ]


@pytest.mark.parametrize("login_on_email_verification", [False, True])
def test_passkey_signup(
    client,
    db,
    webauthn_registration_bypass,
    headless_reverse,
    settings,
    get_last_email_verification_code,
    mailoutbox,
    login_on_email_verification,
):
    settings.ACCOUNT_EMAIL_VERIFICATION = app_settings.EmailVerificationMethod.MANDATORY
    settings.ACCOUNT_EMAIL_VERIFICATION_BY_CODE_ENABLED = True
    # This setting should have no influence when verifying by code:
    settings.ACCOUNT_LOGIN_ON_EMAIL_CONFIRMATION = login_on_email_verification

    # Initiate passkey signup
    resp = client.post(
        headless_reverse("headless:mfa:signup_webauthn"),
        data={"email": "pass@key.org", "username": "passkey"},
        content_type="application/json",
    )

    # Email verification kicks in.
    assert resp.status_code == HTTPStatus.UNAUTHORIZED
    pending_flows = [
        flow for flow in resp.json()["data"]["flows"] if flow.get("is_pending")
    ]
    assert len(pending_flows) == 1
    flow = pending_flows[0]
    assert flow["id"] == Flow.VERIFY_EMAIL.value

    # Verify email.
    code = get_last_email_verification_code(client, mailoutbox)
    resp = client.post(
        headless_reverse("headless:account:verify_email"),
        data={"key": code},
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.UNAUTHORIZED

    # Now, the webauthn signup flow is pending.
    pending_flows = [
        flow for flow in resp.json()["data"]["flows"] if flow.get("is_pending")
    ]
    assert len(pending_flows) == 1
    flow = pending_flows[0]
    assert flow["id"] == Flow.MFA_SIGNUP_WEBAUTHN.value

    # Fetch flow creation options.
    resp = client.get(headless_reverse("headless:mfa:signup_webauthn"))
    data = resp.json()
    assert "creation_options" in data["data"]

    # Create a passkey.
    user = get_user_model().objects.get(email="pass@key.org")
    with webauthn_registration_bypass(user, True) as credential:
        resp = client.put(
            headless_reverse("headless:mfa:signup_webauthn"),
            data={"name": "Some key", "credential": credential},
            content_type="application/json",
        )

    # Signed up successfully.
    data = resp.json()
    assert resp.status_code == HTTPStatus.OK
    assert data["meta"]["is_authenticated"]
    authenticator = Authenticator.objects.get(user=user)
    assert authenticator.wrap().name == "Some key"


def test_get_login_webauthn_doesnt_crash_with_jwt(
    db, client, headless_reverse, settings
):
    settings.HEADLESS_TOKEN_STRATEGY = (
        "tests.apps.headless.tokens.test_jwttokenstrategy.CustomJWTTokenStrategy"
    )
    resp = client.get(headless_reverse("headless:mfa:login_webauthn"))
    assert resp.status_code == HTTPStatus.OK



from django.contrib.auth import BACKEND_SESSION_KEY, HASH_SESSION_KEY, SESSION_KEY
from django.contrib.auth.middleware import AuthenticationMiddleware
from django.contrib.sessions.middleware import SessionMiddleware
from django.http import HttpResponse

from allauth.headless.internal.authkit import purge_request_user_cache


def test_purge_request_user_cache(rf, user):
    request = rf.get("/")
    smw = SessionMiddleware(lambda request: HttpResponse())
    smw(request)
    amw = AuthenticationMiddleware(lambda request: HttpResponse())
    amw(request)
    assert request.user.is_anonymous
    assert not request.user.pk
    purge_request_user_cache(request)
    request.session[SESSION_KEY] = user.pk
    request.session[BACKEND_SESSION_KEY] = (
        "allauth.account.auth_backends.AuthenticationBackend"
    )
    request.session[HASH_SESSION_KEY] = user.get_session_auth_hash()
    assert request.user.is_authenticated
    assert request.user.pk == user.pk



from http import HTTPStatus
from unittest.mock import ANY

from django.urls import reverse

import pytest
from pytest_django.asserts import assertTemplateUsed

from allauth import app_settings as allauth_settings
from allauth.account.adapter import get_adapter
from allauth.account.authentication import AUTHENTICATION_METHODS_SESSION_KEY


@pytest.mark.parametrize(
    "with_totp,with_password,expected_method_urlnames",
    [
        (False, True, ["account_reauthenticate"]),
        (True, True, ["account_reauthenticate", "mfa_reauthenticate"]),
        (True, False, ["mfa_reauthenticate"]),
    ],
)
def test_user_with_mfa_only(
    user_factory, with_totp, with_password, expected_method_urlnames, client
):
    if not allauth_settings.MFA_ENABLED and with_totp:
        return
    user = user_factory(with_totp=with_totp, password=None if with_password else "!")
    assert user.has_usable_password() == with_password
    client.force_login(user)
    methods = get_adapter().get_reauthentication_methods(user)
    assert len(methods) == len(expected_method_urlnames)
    assert {m["url"] for m in methods} == set(map(reverse, expected_method_urlnames))
    for urlname in ["account_reauthenticate", "mfa_reauthenticate"]:
        if urlname == "mfa_reauthenticate" and not allauth_settings.MFA_ENABLED:
            continue
        resp = client.get(f"{reverse(urlname)}?next=/foo")
        if urlname in expected_method_urlnames:
            assert resp.status_code == HTTPStatus.OK
        else:
            assert resp.status_code == HTTPStatus.FOUND
            assert "next=%2Ffoo" in resp["location"]


def test_reauthentication(settings, auth_client, user_password):
    settings.ACCOUNT_REAUTHENTICATION_REQUIRED = True
    resp = auth_client.post(
        reverse("account_email"),
        {"action_add": "", "email": "john3@example.org"},
    )
    assert resp["location"].startswith(reverse("account_reauthenticate"))
    resp = auth_client.get(reverse("account_reauthenticate"))
    assertTemplateUsed(resp, "account/reauthenticate.html")
    resp = auth_client.post(
        reverse("account_reauthenticate"), data={"password": user_password}
    )
    assert resp.status_code == HTTPStatus.FOUND
    resp = auth_client.post(
        reverse("account_email"),
        {"action_add": "", "email": "john3@example.org"},
    )
    assert resp["location"].startswith(reverse("account_email"))
    methods = auth_client.session[AUTHENTICATION_METHODS_SESSION_KEY]
    assert methods[-1] == {"method": "password", "at": ANY, "reauthenticated": True}



from types import SimpleNamespace

import pytest

from allauth.idp.oidc.internal.oauthlib.request_validator import (
    OAuthLibRequestValidator,
)


@pytest.mark.parametrize(
    "origin,allowed_origins,is_allowed",
    [
        ("http://origin", ["https://origin"], False),
        ("https://origin", ["https://origin"], True),
        ("https://origin", [], False),
        ("https://origin", ["https://notthis", "https://origin"], True),
    ],
)
def test_is_origin_allowed(origin, allowed_origins, is_allowed, oidc_client):
    oidc_client.set_cors_origins(allowed_origins)
    oidc_client.save()
    request = SimpleNamespace()
    assert (
        OAuthLibRequestValidator().is_origin_allowed(oidc_client.id, origin, request)
        == is_allowed
    )



from http import HTTPStatus

from django.test import TestCase

from allauth.socialaccount.providers.mailcow.provider import MailcowProvider
from tests.apps.socialaccount.base import OAuth2TestsMixin
from tests.mocking import MockedResponse


class MailcowTests(OAuth2TestsMixin, TestCase):
    provider_id = MailcowProvider.id

    def get_mocked_response(self):
        return MockedResponse(
            HTTPStatus.OK,
            """
        {
            "success": true,
            "username": "user@example.com",
            "id": "user@example.com",
            "identifier": "user@example.com",
            "email": "user@example.com",
            "full_name": "Test User",
            "displayName": "Test User",
            "created": "2021-12-15 14:35:54",
            "modified": "2023-11-02 09:37:58",
            "active": 1
        }
        """,
        )

    def get_expected_to_str(self):
        return "user@example.com"



from rest_framework import authentication, permissions
from rest_framework.response import Response
from rest_framework.views import APIView

from allauth.headless.contrib.rest_framework.authentication import (
    XSessionTokenAuthentication,
)

from .serializers import AddSerializer


class AddAPIView(APIView):
    authentication_classes = [
        authentication.SessionAuthentication,
        XSessionTokenAuthentication,
    ]
    permission_classes = [permissions.IsAuthenticated]

    def get(self, request):
        serializer = AddSerializer(data=request.GET)
        serializer.is_valid(raise_exception=True)
        return Response(
            data={
                "result": serializer.validated_data["x"]
                + serializer.validated_data["y"]
            }
        )



from allauth.socialaccount.providers.base import ProviderAccount
from allauth.socialaccount.providers.lemonldap.views import LemonLDAPOAuth2Adapter
from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider


class LemonLDAPAccount(ProviderAccount):
    def get_avatar_url(self):
        return self.account.extra_data.get("picture")


class LemonLDAPProvider(OAuth2Provider):
    id = "lemonldap"
    name = "LemonLDAP::NG"
    account_class = LemonLDAPAccount
    oauth2_adapter_class = LemonLDAPOAuth2Adapter

    def get_default_scope(self):
        return ["openid", "profile", "email"]

    def extract_uid(self, data):
        return str(data["id"])

    def extract_common_fields(self, data):
        return dict(
            email=data.get("email"),
            username=data.get("preferred_username"),
            name=data.get("name"),
            picture=data.get("picture"),
        )


provider_classes = [LemonLDAPProvider]



from http import HTTPStatus

from django.test import TestCase

from allauth.socialaccount.providers.drip.provider import DripProvider
from tests.apps.socialaccount.base import OAuth2TestsMixin
from tests.mocking import MockedResponse


class DripTests(OAuth2TestsMixin, TestCase):
    provider_id = DripProvider.id

    def get_mocked_response(self):
        return MockedResponse(
            HTTPStatus.OK,
            """{
            "users":[{
                "email": "john@acme.com",
                "name": "John Doe",
                "time_zone": "America/Los_Angeles"
            }]
        }""",
        )

    def get_expected_to_str(self):
        return "john@acme.com"



from django.urls import include, path


urlpatterns = [
    path("idp/", include("tests.projects.common.idp.urls")),
    path("headless/", include("tests.projects.common.headless.urls")),
    path("account/", include("tests.projects.common.account.urls")),
]



from http import HTTPStatus

from django.test import TestCase

from allauth.socialaccount.providers.windowslive.provider import WindowsLiveProvider
from tests.apps.socialaccount.base import OAuth2TestsMixin
from tests.mocking import MockedResponse


class WindowsLiveTests(OAuth2TestsMixin, TestCase):
    provider_id = WindowsLiveProvider.id

    def get_mocked_response(self):
        return MockedResponse(
            HTTPStatus.OK,
            """
        {
          "first_name": "James",
          "last_name": "Smith",
          "name": "James Smith",
          "locale": "en_US",
          "gender": null,
          "emails": {
              "personal": null,
              "account": "jsmith@example.com",
              "business": null,
              "preferred": "jsmith@example.com"
              },
          "link": "https://profile.live.com/",
          "updated_time": "2014-02-07T00:35:27+0000",
          "id": "83605e110af6ff98"
        }
        """,
        )

    def get_expected_to_str(self):
        return "jsmith@example.com"



from http import HTTPStatus

from django.test import TestCase

from allauth.socialaccount.providers.evernote.provider import EvernoteProvider
from tests.apps.socialaccount.base import OAuthTestsMixin
from tests.mocking import MockedResponse


class EvernoteTests(OAuthTestsMixin, TestCase):
    provider_id = EvernoteProvider.id

    def get_mocked_response(self):
        return []

    def get_expected_to_str(self):
        return "Evernote"

    def get_access_token_response(self):
        return MockedResponse(
            HTTPStatus.OK,
            "oauth_token=S%3Ds1%3AU%3D9876%3AE%3D999999b0c50%3AC%3D14c1f89dd18%3AP%3D81%3AA%3Dpennersr%3AV%3D2%3AH%3Ddeadf00dd2d6aba7b519923987b4bf77&oauth_token_secret=&edam_shard=s1&edam_userId=591969&edam_expires=1457994271824&edam_noteStoreUrl=https%3A%2F%2Fsandbox.evernote.com%2Fshard%2Fs1%2Fnotestore&edam_webApiUrlPrefix=https%3A%2F%2Fsandbox.evernote.com%2Fshard%2Fs1%2F",  # noqa
            {"content-type": "text/plain"},
        )



from http import HTTPStatus

from django.test import TestCase

from allauth.socialaccount.providers.robinhood.provider import RobinhoodProvider
from tests.apps.socialaccount.base import OAuth2TestsMixin
from tests.mocking import MockedResponse


class RobinhoodTests(OAuth2TestsMixin, TestCase):
    provider_id = RobinhoodProvider.id

    def get_mocked_response(self):
        return MockedResponse(
            HTTPStatus.OK,
            """
{
  "username": "test_username",
  "id": "1234-5678-910"
}
        """,
        )

    def get_expected_to_str(self):
        return "test_username"



from http import HTTPStatus

from django.test import TestCase

from allauth.socialaccount.providers.questrade.provider import QuestradeProvider
from tests.apps.socialaccount.base import OAuth2TestsMixin
from tests.mocking import MockedResponse


class QuestradeTests(OAuth2TestsMixin, TestCase):
    provider_id = QuestradeProvider.id

    def get_mocked_response(self):
        return MockedResponse(
            HTTPStatus.OK,
            """{"userId":400,"accounts":[]}""",
        )

    def get_expected_to_str(self):
        return "Questrade"



from ninja import NinjaAPI
from ninja.security import django_auth

from allauth.headless.contrib.ninja.security import x_session_token_auth


api = NinjaAPI()


@api.get("/add", auth=[django_auth, x_session_token_auth])
def add(request, x: float, y: float):
    return {"result": x + y}



from http import HTTPStatus

from django.test import TestCase

from allauth.socialaccount.providers.fxa.provider import FirefoxAccountsProvider
from tests.apps.socialaccount.base import OAuth2TestsMixin
from tests.mocking import MockedResponse


class FirefoxAccountsTests(OAuth2TestsMixin, TestCase):
    provider_id = FirefoxAccountsProvider.id

    def get_mocked_response(self):
        return MockedResponse(
            HTTPStatus.OK,
            """
        {
            "uid":"6d940dd41e636cc156074109b8092f96",
            "email":"user@example.com"
        }""",
        )

    def get_expected_to_str(self):
        return "user@example.com"



from django.urls import include, path


urlpatterns = [
    path("drf/", include("tests.projects.common.headless.rest_framework.urls")),
    path("ninja/", include("tests.projects.common.headless.ninja.urls")),
]



from unittest.mock import patch

from allauth.mfa.adapter import get_adapter


def test_build_totp_url_encodes_spaces_as_percent(user):
    """Spaces in the issuer name should be percent-encoded (%20), not encoded as
    '+', so that authenticator apps display the issuer correctly.
    """
    adapter = get_adapter()
    with patch.object(adapter, "get_totp_issuer", return_value="My Company"):
        url = adapter.build_totp_url(user, "JBSWY3DPEHPK3PXP")
    assert "issuer=My%20Company" in url
    assert "issuer=My+Company" not in url



from http import HTTPStatus

from django.conf import settings
from django.contrib.auth import get_user_model
from django.urls import reverse, reverse_lazy

import pytest

from allauth.account.adapter import get_adapter as get_account_adapter
from allauth.socialaccount.models import SocialAccount


@pytest.mark.parametrize(
    "phone_verified, phone_valid, phone_taken, expected_url",
    [
        (True, True, False, settings.LOGIN_REDIRECT_URL),
        (False, True, False, reverse_lazy("account_verify_phone")),
        (True, False, False, reverse_lazy("socialaccount_signup")),
        (True, True, True, reverse_lazy("socialaccount_signup")),
    ],
)
def test_signup_with_phone(
    db,
    settings_impacting_urls,
    client,
    phone,
    phone_verified,
    phone_valid,
    phone_taken,
    expected_url,
    user_factory,
):
    if phone_taken:
        user_factory(phone=phone)
    with settings_impacting_urls(
        SOCIALACCOUNT_AUTO_SIGNUP=True,
        ACCOUNT_LOGIN_METHODS=("phone",),
        ACCOUNT_SIGNUP_FIELDS=["phone*"],
    ):
        resp = client.post(reverse("dummy_login"))
        assert resp.status_code == HTTPStatus.FOUND
        assert resp["location"].startswith(f"{reverse('dummy_authenticate')}?state=")
        resp = client.post(
            resp["location"],
            {
                "id": "123",
                "email": "a@b.com",
                "email_verified": True,
                "phone": phone if phone_valid else "*INVALID*",
                "phone_verified": phone_verified,
            },
        )
        assert resp.status_code == HTTPStatus.FOUND
        assert resp["location"] == expected_url
        if not phone_valid or phone_taken:
            return
        get_user_model().objects.filter(email="a@b.com").exists()
        socialaccount = SocialAccount.objects.get(uid="123")
        account = socialaccount.get_provider_account()
        assert account.to_str() == "a@b.com"
        assert get_account_adapter().get_phone(socialaccount.user) == (
            phone,
            phone_verified,
        )



from http import HTTPStatus
from requests.exceptions import HTTPError

from django.test import TestCase
from django.test.client import RequestFactory
from django.test.utils import override_settings
from django.urls import reverse

from allauth.socialaccount.models import SocialToken
from allauth.socialaccount.providers.ynab.provider import YNABProvider
from tests.apps.socialaccount.base import OAuth2TestsMixin
from tests.mocking import MockedResponse, mocked_response


@override_settings(
    SOCIALACCOUNT_AUTO_SIGNUP=True,
    ACCOUNT_SIGNUP_FORM_CLASS=None,
)
# ACCOUNT_EMAIL_VERIFICATION=account_settings
# .EmailVerificationMethod.MANDATORY)
class YNABTests(OAuth2TestsMixin, TestCase):
    provider_id = YNABProvider.id

    def get_mocked_response(self):
        return MockedResponse(
            HTTPStatus.OK,
            """
              {"data": {
        "user":{
        "id": "abcd1234xyz5678"
                    }
                }
              }
        """,
        )

    def get_expected_to_str(self):
        return "YNAB"

    def test_ynab_compelete_login_401(self):
        from allauth.socialaccount.providers.ynab.views import YNABOAuth2Adapter

        class LessMockedResponse(MockedResponse):
            def raise_for_status(self):
                if self.status_code != HTTPStatus.OK:
                    raise HTTPError(None)

        request = RequestFactory().get(
            reverse(f"{self.provider.id}_login"), dict(process="login")
        )

        adapter = YNABOAuth2Adapter(request)
        app = adapter.get_provider().app
        token = SocialToken(token="some_token")
        response_with_401 = LessMockedResponse(
            HTTPStatus.UNAUTHORIZED,
            """
            {"error": {
              "errors": [{
                "domain": "global",
                "reason": "authError",
                "message": "Invalid Credentials",
                "locationType": "header",
                "location": "Authorization" } ],
              "code": 401,
              "message": "Invalid Credentials" }
            }""",
        )
        with mocked_response(response_with_401):
            with self.assertRaises(HTTPError):
                adapter.complete_login(request, app, token)



from http import HTTPStatus

from django.test import Client
from django.urls import reverse

import pytest

from allauth.usersessions.models import UserSession


def test_overall_flow(user, user_password):
    firefox = Client(HTTP_USER_AGENT="Mozilla Firefox")
    nyxt = Client(HTTP_USER_AGENT="Nyxt")
    for client in [firefox, nyxt]:
        resp = client.post(
            reverse("account_login"),
            {"login": user.username, "password": user_password},
        )
        assert resp.status_code == HTTPStatus.FOUND
    assert UserSession.objects.filter(user=user).count() == 2
    sessions = list(UserSession.objects.filter(user=user).order_by("pk"))
    assert sessions[0].user_agent == "Mozilla Firefox"
    assert sessions[1].user_agent == "Nyxt"
    for client in [firefox, nyxt]:
        resp = client.get(reverse("usersessions_list"))
        assert resp.status_code == HTTPStatus.OK
    resp = firefox.post(reverse("usersessions_list"))
    assert resp.status_code == HTTPStatus.FOUND
    assert UserSession.objects.filter(user=user).count() == 1
    assert UserSession.objects.filter(user=user, pk=sessions[0].pk).exists()
    assert not UserSession.objects.filter(user=user, pk=sessions[1].pk).exists()
    resp = nyxt.get(reverse("usersessions_list"))
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"] == reverse("account_login") + "?next=" + reverse(
        "usersessions_list"
    )


@pytest.mark.parametrize("logout_on_passwd_change", [True, False])
def test_change_password_updates_user_session(
    settings, logout_on_passwd_change, client, user, user_password, password_factory
):
    settings.ACCOUNT_LOGOUT_ON_PASSWORD_CHANGE = logout_on_passwd_change
    resp = client.post(
        reverse("account_login"),
        {"login": user.username, "password": user_password},
    )
    assert resp.status_code == HTTPStatus.FOUND
    assert len(UserSession.objects.purge_and_list(user)) == 1

    new_password = password_factory()
    resp = client.post(
        reverse("account_change_password"),
        {
            "oldpassword": user_password,
            "password1": new_password,
            "password2": new_password,
        },
    )
    assert len(UserSession.objects.purge_and_list(user)) == (
        0 if logout_on_passwd_change else 1
    )



import sys
from http import HTTPStatus

from django.core.exceptions import PermissionDenied
from django.http import HttpResponseRedirect
from django.urls import reverse

import pytest

from allauth.account.adapter import DefaultAccountAdapter
from allauth.core.exceptions import ImmediateHttpResponse


class PreLoginRedirectAccountAdapter(DefaultAccountAdapter):
    def pre_login(self, *args, **kwargs):
        raise ImmediateHttpResponse(HttpResponseRedirect("/foo"))


def test_adapter_pre_login(settings, user, user_password, client):
    settings.ACCOUNT_ADAPTER = (
        "tests.apps.account.test_adapter.PreLoginRedirectAccountAdapter"
    )
    resp = client.post(
        reverse("account_login"),
        {"login": user.username, "password": user_password},
    )
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"] == "/foo"


@pytest.mark.parametrize(
    "x_forwarded_for,expected_ip",
    [
        ("", None),
        ("192.168.1.1", "192.168.1.1"),
        (" 192.168.1.1", "192.168.1.1"),
        ("192.168.1.1 ", "192.168.1.1"),
        (" 192.168.1.1:12345 ", "192.168.1.1"),
        (" [2001:db8::1:0]:12345 ", "2001:db8::1:0"),
        ("192.168.1.1:12345", "192.168.1.1"),
        (
            "2001:0db8:85a3:0000:0000:8a2e:0370:7334",
            "2001:db8:85a3::8a2e:370:7334",
        ),
        (
            "[2001:0db8:85a3:0000:0000:8a2e:0370:7334]:12345",
            "2001:db8:85a3::8a2e:370:7334",
        ),
        ("2001:db8::1:0", "2001:db8::1:0"),
        ("[2001:db8::1:0]:12345", "2001:db8::1:0"),
        ("::abc:7:def", "::abc:7:def"),
        ("::1", "::1"),
        ("[::1]:12345", "::1"),
    ]
    + (
        [
            # not supported by ipaddress py3.8
            ("fe80::1234%1", "fe80::1234%1"),
        ]
        if sys.version_info >= (3, 9)
        else []
    ),
)
def test_get_client_ip_vs_x_forwarded_for(rf, x_forwarded_for, expected_ip, settings):
    settings.ALLAUTH_TRUSTED_PROXY_COUNT = 1
    request = rf.get("/", HTTP_X_FORWARDED_FOR=x_forwarded_for)
    ip = DefaultAccountAdapter(request=request).get_client_ip(request)
    assert ip == (expected_ip if expected_ip else request.META["REMOTE_ADDR"])


@pytest.mark.parametrize(
    "x_forwarded_for",
    [
        " ",
        "999.999.999.999",
        "[999.999.999.999]",
        "999.999.999.999:12345",
        "[999.999.999.999]:12345",
        "[192.0.0.2]",
        "[192.0.0.2]:12345",
        "192.[0].0.2",
        "192.[0].0.2:12345",
        "192.[0.0].2:12345",
        "[192.[0.0].2:12345",
        "2001:db8::1:0:12345",
        "[2001:db8::1:0:12345]",
        "2001:0db8:85a3:0000:0000:8a2e:0370:12345",
        "[2001:0db8:85a3:0000:0000:8a2e:0370]:7334:12345",
        "XwDxNk4JQhUft')) OR 18=(SELECT 18 FROM PG_SLEEP(15))--",
    ],
)
def test_get_client_ip_vs_invalid_x_forwarded_for(rf, x_forwarded_for, settings):
    settings.ALLAUTH_TRUSTED_PROXY_COUNT = 1
    request = rf.get("/", HTTP_X_FORWARDED_FOR=x_forwarded_for)
    with pytest.raises(PermissionDenied):
        DefaultAccountAdapter(request=request).get_client_ip(request)



import json

from django.http import HttpRequest

import pytest

from allauth.core.internal import httpkit


@pytest.mark.parametrize(
    "url,params,expected_url",
    [
        ("/", {"foo": "bar", "v": 1}, "/?foo=bar&v=1"),
        (
            "https://fqdn/?replace=this",
            {"replace": "that"},
            "https://fqdn/?replace=that",
        ),
    ],
)
def test_add_query_params(url, params, expected_url):
    assert httpkit.add_query_params(url, params) == expected_url


@pytest.mark.parametrize(
    "url_template,kwargs,expected_url",
    [
        ("/foo", {}, "http://testserver/foo"),
        ("/foo?key={key}", {"key": " "}, "http://testserver/foo?key=+"),
        ("/foo/{key}", {"key": " "}, "http://testserver/foo/%20"),
        ("https://abs.org/foo?key={key}", {"key": " "}, "https://abs.org/foo?key=+"),
    ],
)
def test_render_url(url_template, kwargs, expected_url, rf):
    request = rf.get("/")
    assert httpkit.render_url(request, url_template, **kwargs) == expected_url


def test_deserialize_request(rf):
    request = rf.get("/")
    assert not request.is_secure()
    serialized = httpkit.serialize_request(request)
    assert not httpkit.deserialize_request(serialized, HttpRequest()).is_secure()
    data = json.loads(serialized)
    data["scheme"] = "https"
    assert httpkit.deserialize_request(json.dumps(data), HttpRequest()).is_secure()



from http import HTTPStatus

from django.urls import reverse

import pytest

from allauth.account.models import EmailAddress


@pytest.fixture(autouse=True)
def prbc_settings(settings_impacting_urls):
    with settings_impacting_urls(ACCOUNT_PASSWORD_RESET_BY_CODE_ENABLED=True):
        yield


@pytest.mark.parametrize("login_on_password_reset", [False, True])
def test_flow(
    user,
    client,
    mailoutbox,
    get_last_password_reset_code,
    password_factory,
    login_on_password_reset,
    settings,
):
    settings.ACCOUNT_LOGIN_ON_PASSWORD_RESET = login_on_password_reset
    new_password = password_factory()
    assert not user.check_password(new_password)
    resp = client.post(reverse("account_reset_password"), {"email": user.email})
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"] == reverse("account_confirm_password_reset_code")
    resp = client.post(
        reverse("account_confirm_password_reset_code"),
        {"code": get_last_password_reset_code(client, mailoutbox)},
    )
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"] == reverse("account_complete_password_reset")
    resp = client.post(
        reverse("account_complete_password_reset"),
        {"password1": new_password, "password2": new_password},
    )
    assert resp.status_code == HTTPStatus.FOUND
    assert (
        resp["location"] == settings.LOGIN_REDIRECT_URL
        if login_on_password_reset
        else reverse("account_password_reset_completed")
    )
    user.refresh_from_db()
    assert user.check_password(new_password)


def test_prevent_enumeration(db, client, mailoutbox):
    resp = client.post(
        reverse("account_reset_password"), {"email": "unknown@account.org"}
    )
    assert resp.status_code == HTTPStatus.FOUND
    assert resp["location"] == reverse("account_confirm_password_reset_code")
    assert mailoutbox[0].subject == "[example.com] Unknown Account"
    resp = client.post(
        reverse("account_confirm_password_reset_code"),
        {"code": "none?"},
    )
    assert resp.status_code == HTTPStatus.OK


def test_indirect_email_verification(
    db, client, mailoutbox, user_factory, get_last_password_reset_code
):
    user = user_factory(email_verified=False)
    address = EmailAddress.objects.get(user=user, email=user.email, verified=False)
    resp = client.post(reverse("account_reset_password"), {"email": address.email})
    assert resp.status_code == HTTPStatus.FOUND
    resp = client.post(
        reverse("account_confirm_password_reset_code"),
        {"code": get_last_password_reset_code(client, mailoutbox)},
    )
    assert resp.status_code == HTTPStatus.FOUND
    address.refresh_from_db()
    assert address.verified



from http import HTTPStatus

from django.test.client import Client
from django.urls import reverse, reverse_lazy

import jwt
import pytest

from allauth.headless.tokens.strategies.jwt import JWTTokenStrategy


class CustomJWTTokenStrategy(JWTTokenStrategy):
    def get_claims(self, user):
        return {"email": user.email}


@pytest.fixture
def custom_jwt_token_strategy(settings):
    settings.HEADLESS_TOKEN_STRATEGY = (
        "tests.apps.headless.tokens.test_jwttokenstrategy.CustomJWTTokenStrategy"
    )


@pytest.fixture
def obtain_tokens(user, user_password, headless_reverse):
    def f(client):
        # Let's sign in
        resp = client.post(
            headless_reverse("headless:account:login"),
            data={
                "username": user.username,
                "password": user_password,
            },
            content_type="application/json",
        )
        assert resp.status_code == HTTPStatus.OK

        # On sign in, we receive an access/refresh token pair.
        meta = resp.json()["meta"]
        assert meta["is_authenticated"]
        access_token = meta.get("access_token")
        refresh_token = meta.get("refresh_token")
        return access_token, refresh_token

    return f


@pytest.mark.parametrize("rotate", [False, True])
def test_rotate_refresh_token(
    headless_client,
    headless_reverse,
    client,
    user,
    rotate,
    settings,
    obtain_tokens,
    custom_jwt_token_strategy,
):
    if headless_client == "browser":
        return
    settings.HEADLESS_JWT_ROTATE_REFRESH_TOKEN = rotate

    access_token, refresh_token = obtain_tokens(client)

    # Check our custom claim
    access_token_data = jwt.decode(
        access_token, algorithms=["RS256"], options={"verify_signature": False}
    )
    assert access_token_data["email"] == user.email

    # Let's refresh
    resp = Client().post(
        headless_reverse("headless:tokens:refresh"),
        data={"refresh_token": refresh_token},
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.OK
    data = resp.json()["data"]

    # Check our custom claim in the new access token
    access_token = data["access_token"]
    access_token_data = jwt.decode(
        access_token, algorithms=["RS256"], options={"verify_signature": False}
    )
    assert access_token_data["email"] == user.email

    new_refresh_token = data.get("refresh_token")
    assert bool(new_refresh_token) == rotate

    # Let's refresh, again using the previous refresh token
    resp = Client().post(
        headless_reverse("headless:tokens:refresh"),
        data={"refresh_token": refresh_token},
        content_type="application/json",
    )
    assert resp.status_code == (HTTPStatus.BAD_REQUEST if rotate else HTTPStatus.OK)

    # Let's refresh, using the new refresh token
    if new_refresh_token:
        resp = Client().post(
            headless_reverse("headless:tokens:refresh"),
            data={"refresh_token": new_refresh_token},
            content_type="application/json",
        )
        assert resp.status_code == HTTPStatus.OK


@pytest.mark.parametrize("stateful_validation_enabled", [False, True])
def test_flow(
    headless_client,
    headless_reverse,
    client,
    user,
    settings,
    stateful_validation_enabled,
    obtain_tokens,
):
    settings.HEADLESS_JWT_ROTATE_REFRESH_TOKEN = False
    settings.HEADLESS_TOKEN_STRATEGY = (
        "allauth.headless.tokens.strategies.jwt.JWTTokenStrategy"
    )
    settings.HEADLESS_JWT_STATEFUL_VALIDATION_ENABLED = stateful_validation_enabled

    access_token, refresh_token = obtain_tokens(client)

    # On sign in, we receive an access/refresh token pair.
    if headless_client == "browser":
        assert not access_token
        assert not refresh_token
        return

    # With the access token, we can reach out to the allauth API.
    at_client = Client(HTTP_AUTHORIZATION=f"Bearer {access_token}")
    resp = at_client.get(headless_reverse("headless:account:current_session"))
    assert resp.status_code == HTTPStatus.OK

    # Also check validity with DRF & Ninja endpoints
    for url in [
        reverse("headless_rest_framework_resource"),
        "/headless/ninja/resource",
    ]:
        resp = Client(HTTP_AUTHORIZATION=f"Bearer {access_token}").get(
            f"{url}?userinfo"
        )
        assert resp.status_code == HTTPStatus.OK
        assert resp.json() == {"resource": "ok", "user_email": user.email}

    # With the refresh token, we can retrieve a new access token.
    resp = Client().post(
        headless_reverse("headless:tokens:refresh"),
        data={"refresh_token": refresh_token},
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.OK
    new_access_token = resp.json()["data"]["access_token"]
    assert new_access_token != access_token

    # And, of course, that new access token is valid.
    at_client = Client(HTTP_AUTHORIZATION=f"Bearer {new_access_token}")
    resp = at_client.get(headless_reverse("headless:account:current_session"))
    assert resp.status_code == HTTPStatus.OK

    # Also check validity with DRF & Ninja endpoints
    for url in [
        reverse("headless_rest_framework_resource"),
        "/headless/ninja/resource",
    ]:
        resp = Client(HTTP_AUTHORIZATION=f"Bearer {access_token}").get(
            f"{url}?userinfo"
        )
        assert resp.status_code == HTTPStatus.OK
        assert resp.json() == {"resource": "ok", "user_email": user.email}

    # But, when we logout...
    resp = at_client.delete(headless_reverse("headless:account:current_session"))
    assert resp.status_code == HTTPStatus.UNAUTHORIZED

    # ... the refresh token no longer works.
    resp = Client().post(
        headless_reverse("headless:tokens:refresh"),
        data={"refresh_token": refresh_token},
        content_type="application/json",
    )
    assert resp.status_code == HTTPStatus.BAD_REQUEST

    # And, the access token no longer functions...
    at_client = Client(HTTP_AUTHORIZATION=f"Bearer {new_access_token}")
    resp = at_client.get(headless_reverse("headless:account:current_session"))
    assert resp.status_code == (
        HTTPStatus.UNAUTHORIZED if stateful_validation_enabled else HTTPStatus.GONE
    )

    # Also check validity with DRF & Ninja endpoints
    for url in [
        reverse("headless_rest_framework_resource"),
        "/headless/ninja/resource",
    ]:
        resp = Client(HTTP_AUTHORIZATION=f"Bearer {access_token}").get(url)
        assert resp.status_code == (
            HTTPStatus.UNAUTHORIZED if stateful_validation_enabled else HTTPStatus.OK
        )


@pytest.mark.parametrize("stateful,query_count", [(False, 0), (True, 1)])
@pytest.mark.parametrize(
    "url",
    [
        reverse_lazy("headless_rest_framework_resource"),
        "/headless/ninja/resource",
    ],
)
def test_access_token_query_counts(
    headless_client,
    headless_reverse,
    client,
    settings,
    obtain_tokens,
    django_assert_num_queries,
    stateful,
    query_count,
    url,
):
    if headless_client == "browser":
        return
    settings.HEADLESS_TOKEN_STRATEGY = (
        "allauth.headless.tokens.strategies.jwt.JWTTokenStrategy"
    )
    settings.HEADLESS_JWT_STATEFUL_VALIDATION_ENABLED = stateful
    access_token, _ = obtain_tokens(client)
    with django_assert_num_queries(query_count):
        resp = Client(HTTP_AUTHORIZATION=f"Bearer {access_token}").get(url)
        assert resp.status_code == (HTTPStatus.OK)


@pytest.mark.parametrize("settings_scheme", ["Bearer", "Token", "JWT"])
@pytest.mark.parametrize("request_scheme", ["Bearer", "Token", "JWT"])
def test_custom_authorization_header_scheme(
    headless_client,
    headless_reverse,
    client,
    settings,
    obtain_tokens,
    settings_scheme,
    request_scheme,
):
    """Test that JWT_AUTHORIZATION_HEADER_SCHEME setting is respected."""
    if headless_client == "browser":
        return

    # Set authorization header scheme in settings
    settings.HEADLESS_JWT_AUTHORIZATION_HEADER_SCHEME = settings_scheme
    settings.HEADLESS_TOKEN_STRATEGY = (
        "allauth.headless.tokens.strategies.jwt.JWTTokenStrategy"
    )

    access_token, _ = obtain_tokens(client)

    # Make request with the specified scheme
    at_client = Client(HTTP_AUTHORIZATION=f"{request_scheme} {access_token}")
    resp = at_client.get(headless_reverse("headless:account:current_session"))

    # Should succeed only when request scheme matches settings scheme
    if settings_scheme == request_scheme:
        assert resp.status_code == HTTPStatus.OK
    else:
        assert resp.status_code == HTTPStatus.UNAUTHORIZED


def test_hs256_algorithm(
    headless_client,
    headless_reverse,
    client,
    settings,
    user,
    obtain_tokens,
):
    if headless_client == "browser":
        return
    settings.HEADLESS_JWT_ALGORITHM = "HS256"
    settings.HEADLESS_JWT_PRIVATE_KEY = "super-secret"
    settings.HEADLESS_TOKEN_STRATEGY = (
        "allauth.headless.tokens.strategies.jwt.JWTTokenStrategy"
    )

    access_token, _ = obtain_tokens(client)

    header = jwt.get_unverified_header(access_token)
    assert header["alg"] == "HS256"
    assert "kid" not in header

    payload = jwt.decode(
        access_token,
        key="super-secret",
        algorithms=["HS256"],
        options={"verify_signature": True, "verify_iss": False, "verify_aud": False},
    )
    assert payload["sub"] == str(user.pk)

    at_client = Client(HTTP_AUTHORIZATION=f"Bearer {access_token}")
    resp = at_client.get(headless_reverse("headless:account:current_session"))
    assert resp.status_code == HTTPStatus.OK


def test_hs256_fallback_to_secret_key(
    headless_client,
    headless_reverse,
    client,
    settings,
    user,
    obtain_tokens,
):
    if headless_client == "browser":
        return
    settings.HEADLESS_JWT_ALGORITHM = "HS256"
    settings.HEADLESS_JWT_PRIVATE_KEY = ""  # Empty
    settings.SECRET_KEY = "django-secret-fallback"
    settings.HEADLESS_TOKEN_STRATEGY = (
        "allauth.headless.tokens.strategies.jwt.JWTTokenStrategy"
    )

    access_token, _ = obtain_tokens(client)

    payload = jwt.decode(
        access_token,
        key="django-secret-fallback",
        algorithms=["HS256"],
        options={"verify_signature": True, "verify_iss": False, "verify_aud": False},
    )
    assert payload["sub"] == str(user.pk)



from django.core.exceptions import ValidationError

import pytest

from allauth.idp.oidc.internal.clientkit import (
    _wildcard_to_regex,
    is_origin_allowed,
    is_redirect_uri_allowed,
)
from allauth.idp.oidc.models import Client


URI_TEST_CASES = [
    (False, "https://example.com", True),
    (False, "https://*.example.com", False),
    (True, "https://example.com", True),
    (True, "https://*.projectname.pages.dev/", True),
    (True, "https://*--projectname-com.netlify.app", True),
    (True, "http://*.localhost:3000", True),
    (True, "https://example.com/callback", True),
    (True, "https://*.*.example.com", False),
    (True, "http*://example.com", False),
    (True, "https://example.com/callback/*", False),
]


@pytest.mark.parametrize(
    ("allow_wildcards", "uri", "should_pass"),
    URI_TEST_CASES,
)
def test_redirect_uri_wildcard_validation(allow_wildcards, uri, should_pass):
    client = Client(
        id="test-client",
        name="Test Client",
        allow_uri_wildcards=allow_wildcards,
        redirect_uris=uri,
    )

    if should_pass:
        client.clean_redirect_uris()
    else:
        with pytest.raises(ValidationError):
            client.clean_redirect_uris()


@pytest.mark.parametrize(
    ("allow_wildcards", "origin", "should_pass"),
    URI_TEST_CASES,
)
def test_cors_origin_wildcard_validation(allow_wildcards, origin, should_pass):
    client = Client(
        id="test-client",
        name="Test Client",
        allow_uri_wildcards=allow_wildcards,
        cors_origins=origin,
    )

    if should_pass:
        client.clean_cors_origins()
    else:
        with pytest.raises(ValidationError):
            client.clean_cors_origins()


def test_wildcards_disabled_by_default():
    client = Client(
        id="test-client",
        name="Test Client",
        redirect_uris="https://example.com",
    )

    assert client.allow_uri_wildcards is False

    client.redirect_uris = "https://*.example.com"
    with pytest.raises(ValidationError):
        client.clean_redirect_uris()


def test_wildcard_matching_enforcement():
    client = Client(
        id="test-client",
        name="Test Client",
        allow_uri_wildcards=True,
        redirect_uris="https://*.example.com/callback",
        cors_origins="https://*.example.com",
    )

    assert is_redirect_uri_allowed(
        "https://subdomain.example.com/callback",
        client.get_redirect_uris(),
        client.allow_uri_wildcards,
    )
    assert not is_redirect_uri_allowed(
        "https://evil-example.com/callback",
        client.get_redirect_uris(),
        client.allow_uri_wildcards,
    )

    assert is_origin_allowed(
        "https://subdomain.example.com",
        client.get_cors_origins(),
        client.allow_uri_wildcards,
    )

    assert not is_origin_allowed(
        "https://evil-example.com",
        client.get_cors_origins(),
        client.allow_uri_wildcards,
    )

    assert not is_origin_allowed(
        "https://user:pass@foo.example.com",
        client.get_cors_origins(),
        client.allow_uri_wildcards,
    )


def test_wildcard_to_regex():
    pattern = _wildcard_to_regex("*.example.com")
    assert pattern.pattern == r"^[^.]+\.example\.com$"

    assert pattern.match("api.example.com")
    assert not pattern.match("foo.api.example.com")  # * should not match dots
    assert not pattern.match("evil-example.com")

    # verify dots aren't treated as wildcards
    assert not pattern.match("api$example$com")



from http import HTTPStatus

from django.test import TestCase

from allauth.socialaccount.providers.baidu.provider import BaiduProvider
from tests.apps.socialaccount.base import OAuth2TestsMixin
from tests.mocking import MockedResponse


class BaiduTests(OAuth2TestsMixin, TestCase):
    provider_id = BaiduProvider.id

    def get_mocked_response(self):
        return MockedResponse(
            HTTPStatus.OK,
            """
{"portrait": "78c0e9839de59bbde7859ccf43",
"uname": "\u90dd\u56fd\u715c", "uid": "3225892368"}""",
        )

    def get_expected_to_str(self):
        return "\u90dd\u56fd\u715c"
